@@ -2,23 +2,13 @@
 
 # (c) 2012, Mark Theunissen <mark.theunissen@gmail.com>
 # Sponsored by Four Kitchens http://fourkitchens.com.
-#
-# This file is part of Ansible
-#
-# Ansible is free software: you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# Ansible is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-#
-# You should have received a copy of the GNU General Public License
-# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
+# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
 
-ANSIBLE_METADATA = {'metadata_version': '1.0',
+from __future__ import absolute_import, division, print_function
+__metaclass__ = type
+
+
+ANSIBLE_METADATA = {'metadata_version': '1.1',
                     'status': ['preview'],
                     'supported_by': 'community'}
 
@@ -214,17 +204,23 @@ EXAMPLES = """
 # password=n<_665{vS43y
 """
 
-import getpass
-import tempfile
 import re
 import string
+import traceback
+
 try:
     import MySQLdb
 except ImportError:
     mysqldb_found = False
 else:
     mysqldb_found = True
+
+from ansible.module_utils.basic import AnsibleModule
+from ansible.module_utils.database import SQLParseError
+from ansible.module_utils.mysql import mysql_connect, mysqldb_found
 from ansible.module_utils.six import iteritems
+from ansible.module_utils._text import to_native
+
 
 VALID_PRIVS = frozenset(('CREATE', 'DROP', 'GRANT', 'GRANT OPTION',
                          'LOCK TABLES', 'REFERENCES', 'EVENT', 'ALTER',
@@ -236,6 +232,7 @@ VALID_PRIVS = frozenset(('CREATE', 'DROP', 'GRANT', 'GRANT OPTION',
                          'REPLICATION SLAVE', 'SHOW DATABASES', 'SHUTDOWN',
                          'SUPER', 'ALL', 'ALL PRIVILEGES', 'USAGE', 'REQUIRESSL'))
 
+
 class InvalidPrivsError(Exception):
     pass
 
@@ -243,6 +240,7 @@ class InvalidPrivsError(Exception):
 # MySQL module specific support methods.
 #
 
+
 # User Authentication Management was change in MySQL 5.7
 # This is a generic check for if the server version is less than version 5.7
 def server_version_check(cursor):
@@ -255,11 +253,12 @@ def server_version_check(cursor):
     # mariadb and the old-style update continues to work
     if 'mariadb' in version_str.lower():
         return True
-    if (int(version[0]) <= 5 and int(version[1]) < 7):
+    if int(version[0]) <= 5 and int(version[1]) < 7:
         return True
     else:
         return False
 
+
 def get_mode(cursor):
     cursor.execute('SELECT @@GLOBAL.sql_mode')
     result = cursor.fetchone()
@@ -270,15 +269,17 @@ def get_mode(cursor):
         mode = 'NOTANSI'
     return mode
 
+
 def user_exists(cursor, user, host, host_all):
     if host_all:
         cursor.execute("SELECT count(*) FROM user WHERE user = %s", ([user]))
     else:
-        cursor.execute("SELECT count(*) FROM user WHERE user = %s AND host = %s", (user,host))
+        cursor.execute("SELECT count(*) FROM user WHERE user = %s AND host = %s", (user, host))
 
     count = cursor.fetchone()
     return count[0] > 0
 
+
 def user_add(cursor, user, host, host_all, password, encrypted, new_priv, check_mode):
     # we cannot create users without a proper hostname
     if host_all:
@@ -288,16 +289,17 @@ def user_add(cursor, user, host, host_all, password, encrypted, new_priv, check_
         return True
 
     if password and encrypted:
-        cursor.execute("CREATE USER %s@%s IDENTIFIED BY PASSWORD %s", (user,host,password))
+        cursor.execute("CREATE USER %s@%s IDENTIFIED BY PASSWORD %s", (user, host, password))
     elif password and not encrypted:
-        cursor.execute("CREATE USER %s@%s IDENTIFIED BY %s", (user,host,password))
+        cursor.execute("CREATE USER %s@%s IDENTIFIED BY %s", (user, host, password))
     else:
-        cursor.execute("CREATE USER %s@%s", (user,host))
+        cursor.execute("CREATE USER %s@%s", (user, host))
     if new_priv is not None:
         for db_table, priv in iteritems(new_priv):
-            privileges_grant(cursor, user,host,db_table,priv)
+            privileges_grant(cursor, user, host, db_table, priv)
     return True
 
+
 def is_hash(password):
     ishash = False
     if len(password) == 41 and password[0] == '*':
@@ -305,6 +307,7 @@ def is_hash(password):
             ishash = True
     return ishash
 
+
 def user_mod(cursor, user, host, host_all, password, encrypted, new_priv, append_privs, module):
     changed = False
     grant_option = False
@@ -321,9 +324,9 @@ def user_mod(cursor, user, host, host_all, password, encrypted, new_priv, append
             old_user_mgmt = server_version_check(cursor)
 
             if old_user_mgmt:
-                cursor.execute("SELECT password FROM user WHERE user = %s AND host = %s", (user,host))
+                cursor.execute("SELECT password FROM user WHERE user = %s AND host = %s", (user, host))
             else:
-                cursor.execute("SELECT authentication_string FROM user WHERE user = %s AND host = %s", (user,host))
+                cursor.execute("SELECT authentication_string FROM user WHERE user = %s AND host = %s", (user, host))
             current_pass_hash = cursor.fetchone()
 
             if encrypted:
@@ -356,7 +359,7 @@ def user_mod(cursor, user, host, host_all, password, encrypted, new_priv, append
 
         # Handle privileges
         if new_priv is not None:
-            curr_priv = privileges_get(cursor, user,host)
+            curr_priv = privileges_get(cursor, user, host)
 
             # If the user has privileges on a db.table that doesn't appear at all in
             # the new specification, then revoke all privileges on it.
@@ -368,7 +371,7 @@ def user_mod(cursor, user, host, host_all, password, encrypted, new_priv, append
                     if user != "root" and "PROXY" not in priv and not append_privs:
                         if module.check_mode:
                             return True
-                        privileges_revoke(cursor, user,host,db_table,priv,grant_option)
+                        privileges_revoke(cursor, user, host, db_table, priv, grant_option)
                         changed = True
 
             # If the user doesn't currently have any privileges on a db.table, then
@@ -377,7 +380,7 @@ def user_mod(cursor, user, host, host_all, password, encrypted, new_priv, append
                 if db_table not in curr_priv:
                     if module.check_mode:
                         return True
-                    privileges_grant(cursor, user,host,db_table,priv)
+                    privileges_grant(cursor, user, host, db_table, priv)
                     changed = True
 
             # If the db.table specification exists in both the user's current privileges
@@ -385,16 +388,17 @@ def user_mod(cursor, user, host, host_all, password, encrypted, new_priv, append
             db_table_intersect = set(new_priv.keys()) & set(curr_priv.keys())
             for db_table in db_table_intersect:
                 priv_diff = set(new_priv[db_table]) ^ set(curr_priv[db_table])
-                if (len(priv_diff) > 0):
+                if len(priv_diff) > 0:
                     if module.check_mode:
                         return True
                     if not append_privs:
-                        privileges_revoke(cursor, user,host,db_table,curr_priv[db_table],grant_option)
-                    privileges_grant(cursor, user,host,db_table,new_priv[db_table])
+                        privileges_revoke(cursor, user, host, db_table, curr_priv[db_table], grant_option)
+                    privileges_grant(cursor, user, host, db_table, new_priv[db_table])
                     changed = True
 
     return changed
 
+
 def user_delete(cursor, user, host, host_all, check_mode):
     if check_mode:
         return True
@@ -409,6 +413,7 @@ def user_delete(cursor, user, host, host_all, check_mode):
 
     return True
 
+
 def user_get_hostnames(cursor, user):
     cursor.execute("SELECT Host FROM mysql.user WHERE user = %s", user)
     hostnames_raw = cursor.fetchall()
@@ -419,7 +424,8 @@ def user_get_hostnames(cursor, user):
 
     return hostnames
 
-def privileges_get(cursor, user,host):
+
+def privileges_get(cursor, user, host):
     """ MySQL doesn't have a better method of getting privileges aside from the
     SHOW GRANTS query syntax, which requires us to then parse the returned string.
     Here's an example of the string that is returned from MySQL:
@@ -440,11 +446,11 @@ def privileges_get(cursor, user,host):
             return x
 
     for grant in grants:
-        res = re.match("GRANT (.+) ON (.+) TO '.*'@'.+'( IDENTIFIED BY PASSWORD '.+')? ?(.*)", grant[0])
+        res = re.match("GRANT (.+) ON (.+) TO '.*'@'.*'( IDENTIFIED BY PASSWORD '.+')? ?(.*)", grant[0])
         if res is None:
             raise InvalidPrivsError('unable to parse the MySQL grant string: %s' % grant[0])
         privileges = res.group(1).split(", ")
-        privileges = [ pick(x) for x in privileges]
+        privileges = [pick(x) for x in privileges]
         if "WITH GRANT OPTION" in res.group(4):
             privileges.append('GRANT')
         if "REQUIRE SSL" in res.group(4):
@@ -453,6 +459,7 @@ def privileges_get(cursor, user,host):
         output[db] = privileges
     return output
 
+
 def privileges_unpack(priv, mode):
     """ Take a privileges string, typically passed as a parameter, and unserialize
     it into a dictionary, the same format as privileges_get() above. We have this
@@ -483,7 +490,7 @@ def privileges_unpack(priv, mode):
         if '(' in pieces[1]:
             output[pieces[0]] = re.split(r',\s*(?=[^)]*(?:\(|$))', pieces[1].upper())
             for i in output[pieces[0]]:
-                privs.append(re.sub(r'\s*\(.*\)','',i))
+                privs.append(re.sub(r'\s*\(.*\)', '', i))
         else:
             output[pieces[0]] = pieces[1].upper().split(',')
             privs = output[pieces[0]]
@@ -501,7 +508,8 @@ def privileges_unpack(priv, mode):
 
     return output
 
-def privileges_revoke(cursor, user,host,db_table,priv,grant_option):
+
+def privileges_revoke(cursor, user, host, db_table, priv, grant_option):
     # Escape '%' since mysql db.execute() uses a format string
     db_table = db_table.replace('%', '%%')
     if grant_option:
@@ -515,7 +523,8 @@ def privileges_revoke(cursor, user,host,db_table,priv,grant_option):
     query = ' '.join(query)
     cursor.execute(query, (user, host))
 
-def privileges_grant(cursor, user,host,db_table,priv):
+
+def privileges_grant(cursor, user, host, db_table, priv):
     # Escape '%' since mysql db.execute uses a format string and the
     # specification of db and table often use a % (SQL wildcard)
     db_table = db_table.replace('%', '%%')
@@ -533,9 +542,10 @@ def privileges_grant(cursor, user,host,db_table,priv):
 # Module execution.
 #
 
+
 def main():
     module = AnsibleModule(
-        argument_spec = dict(
+        argument_spec=dict(
             login_user=dict(default=None),
             login_password=dict(default=None, no_log=True),
             login_host=dict(default="localhost"),
@@ -581,7 +591,7 @@ def main():
     sql_log_bin = module.params["sql_log_bin"]
 
     if not mysqldb_found:
-        module.fail_json(msg="the python mysqldb module is required")
+        module.fail_json(msg="The MySQL-python module is required.")
 
     cursor = None
     try:
@@ -595,9 +605,9 @@ def main():
         if not cursor:
             cursor = mysql_connect(module, login_user, login_password, config_file, ssl_cert, ssl_key, ssl_ca, db,
                                    connect_timeout=connect_timeout)
-    except Exception:
-        e = get_exception()
-        module.fail_json(msg="unable to connect to database, check login_user and login_password are correct or %s has the credentials. Exception message: %s" % (config_file, e))
+    except Exception as e:
+        module.fail_json(msg="unable to connect to database, check login_user and login_password are correct or %s has the credentials. "
+                             "Exception message: %s" % (config_file, to_native(e)))
 
     if not sql_log_bin:
         cursor.execute("SET SQL_LOG_BIN=0;")
@@ -605,14 +615,12 @@ def main():
     if priv is not None:
         try:
             mode = get_mode(cursor)
-        except Exception:
-            e = get_exception()
-            module.fail_json(msg=str(e))
+        except Exception as e:
+            module.fail_json(msg=to_native(e), exception=traceback.format_exc())
         try:
             priv = privileges_unpack(priv, mode)
-        except Exception:
-            e = get_exception()
-            module.fail_json(msg="invalid privileges string: %s" % str(e))
+        except Exception as e:
+            module.fail_json(msg="invalid privileges string: %s" % to_native(e))
 
     if state == "present":
         if user_exists(cursor, user, host, host_all):
@@ -622,17 +630,15 @@ def main():
                 else:
                     changed = user_mod(cursor, user, host, host_all, None, encrypted, priv, append_privs, module)
 
-            except (SQLParseError, InvalidPrivsError, MySQLdb.Error):
-                e = get_exception()
-                module.fail_json(msg=str(e))
+            except (SQLParseError, InvalidPrivsError, MySQLdb.Error) as e:
+                module.fail_json(msg=to_native(e), exception=traceback.format_exc())
         else:
             if host_all:
                 module.fail_json(msg="host_all parameter cannot be used when adding a user")
             try:
                 changed = user_add(cursor, user, host, host_all, password, encrypted, priv, module.check_mode)
-            except (SQLParseError, InvalidPrivsError, MySQLdb.Error):
-                e = get_exception()
-                module.fail_json(msg=str(e))
+            except (SQLParseError, InvalidPrivsError, MySQLdb.Error) as e:
+                module.fail_json(msg=to_native(e), exception=traceback.format_exc())
     elif state == "absent":
         if user_exists(cursor, user, host, host_all):
             changed = user_delete(cursor, user, host, host_all, module.check_mode)
@@ -640,9 +646,6 @@ def main():
             changed = False
     module.exit_json(changed=changed, user=user)
 
-# import module snippets
-from ansible.module_utils.basic import *
-from ansible.module_utils.database import *
-from ansible.module_utils.mysql import *
+
 if __name__ == '__main__':
     main()
